#####################################################################
# EJ(20190319)
# MED indicators
#####################################################################

library(ggplot2)
library(lme4)
library(influence.ME)
library(lattice)
library(parallel)
library(rgdal)
library(reshape2)
library(plyr)
source("funs.R")

#====================================================================
# Setup
#====================================================================

# year when assessments were performed 
assessmentYear <- 2019
# final year with estimations from stock assessments 
fnlYear <- assessmentYear - 1
# initial year with estimations from stock assessments 
iniYear <- 2003
# vector of years
dy <- iniYear:fnlYear
# vector of years for valid assessments
vay <- (assessmentYear-2):assessmentYear
# vector of years for stock status projection
vpy <- (fnlYear-2):fnlYear
# options for reading data
options(stringsAsFactors=FALSE)
# number of simulations for mle bootstrap
it <- 500
# number of cores for mle bootstrap parallel
nc <- 3
# quantiles to be computed
qtl <- c(0.025, 0.25, 0.50, 0.75, 0.975)
# to control de seed in mclapply
RNGkind("L'Ecuyer-CMRG")
set.seed(1234)
# to make plots consistent
vp <- dy
vp[c(2,4,6,8,10,12,14)] <- ""
theme_set(theme_bw())
sc <- scale_x_continuous(breaks=dy, labels=as.character(vp))
th <- theme(axis.text.x  = element_text(angle=90, vjust=0.5), panel.grid.minor = element_blank())

#====================================================================
# load & pre-process
#====================================================================

#--------------------------------------------------------------------
# load and pre-process
#--------------------------------------------------------------------

# assessments
gfcm <- read.csv("../data/med/GFCM_SA_2020.csv")
gfcm$Meeting <- "GFCM"
#gfcm$Fref <- gfcm$Fref_point
stecf <- read.csv("../data/med/STECF_CFP_2020.csv")
msa <- rbind(stecf, gfcm)
msa$Fref <- msa$Fref_point

# keep relevant columns only 
msa <- msa[,c("Stock", "Area", "Year", "R", "SSB", "F", "Fref", "Blim", "Bref", "asses_year", "Meeting", "Assessment_URL", "Species", "EcoRegion")]

# id assessment source 
msa[msa$Meeting!="GFCM","Meeting"] <- "STECF"
names(msa)[names(msa)=="Meeting"] <- "source"

#--------------------------------------------------------------------
# recode and compute indicators
#--------------------------------------------------------------------
msa$stk <- tolower(paste(msa$Stock, msa$Area, sep="_"))
msa$StockDescription <- paste(msa$Species, "in GSA", gsub("_", ", ",  msa$Area))
msa$Fref <- as.numeric(msa$Fref)
msa <- transform(msa, indF = F/Fref)
msa <- transform(msa, sfFind=!is.na(indF), i1=indF>1, i2=indF<=1)

#--------------------------------------------------------------------
# subset 
# (filtering through the sampling frame done during data harvesting)
#--------------------------------------------------------------------
sam <- msa[!is.na(msa$indF) & msa$Year >=iniYear & msa$Year <= fnlYear & msa$asses_year %in% vay,]

#--------------------------------------------------------------------
# project stock status 
# (check fnlYear < assessmentYear-1)
#--------------------------------------------------------------------
sam$projected <- FALSE

# use y-2 for stocks missing in y-1
sy2 <- sam[sam$Year==sort(vpy)[1], "stk"]
sy1 <- sam[sam$Year==sort(vpy)[2], "stk"]
v0 <- sy2[!(sy2 %in% sy1)]
if(length(v0)>0){
	df0 <- subset(sam, Year==sort(vpy)[1] & stk %in% v0)
	df0$Year <- sort(vpy)[2]
	df0$projected <- TRUE
	sam <- rbind(sam, df0)
}

# use y-1 for stocks missing in y
sy <- sam[sam$Year==sort(vpy)[3], "stk"]
v0 <- sy1[!(sy1 %in% sy)]
if(length(v0)>0){
	df0 <- subset(sam, Year==sort(vpy)[2] & stk %in% v0)
	df0$Year <- sort(vpy)[3]
	df0$projected <- TRUE
	sam <- rbind(sam, df0)
}

#====================================================================
# Indicators
#====================================================================
#--------------------------------------------------------------------
# Number of stocks (remove projected years)
#--------------------------------------------------------------------
df0 <- sam[!sam$projected,]
mnStks <- aggregate(stk~Year, df0, length)
names(mnStks) <- c("Year", "N")

# plot
png("figMedI0.png", 1800, 1200, res=300)
ggplot(subset(mnStks, Year!=fnlYear), aes(x=Year, y=N)) + 
	geom_line() + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,55)) + 
	sc + 
	th +
    geom_point(aes(x=fnlYear, y=mnStks$N[length(mnStks$N)]), size=2)
dev.off()

png("figMedI0b.png", 1200, 1600, res=200)
df0 <- sam[!sam$projected,]
ggplot(df0, aes(Year,reorder(stk, desc(stk))))+ 
	geom_line() +
	geom_point(data=aggregate(list(Year=df0$Year, EcoRegion=df0$EcoRegion),
    by=list(stk=df0$stk), max)) +
	ylab("") +
	xlab("Year") +
	sc +
	th +
	facet_grid(EcoRegion~., switch="y", space="free_y", scales="free_y") +
	theme(strip.placement="outside", strip.background.y=element_blank(),
	panel.spacing.y=unit(0.05, "lines"))
dev.off()

write.csv(dcast(df0, EcoRegion~Year, value.var='stk', margins=TRUE, fun.aggregate=length), file="tabMedI0.csv", row.names=FALSE) 

#--------------------------------------------------------------------
#  drop final assessment year, redo scales for plotting
#--------------------------------------------------------------------
sam <- sam[sam$Year!=fnlYear,]

vp <- iniYear:I(fnlYear-1)
vp[c(2,4,6,8,10,12,14)] <- ""
sc <- scale_x_continuous(breaks=iniYear:I(fnlYear-1), labels=as.character(vp))

#--------------------------------------------------------------------
# (I7) F/Fmsy model based indicator
#--------------------------------------------------------------------
df0 <- sam
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))
No <- length(unique(df0$stk))

# model
mfit <- glmer(indF ~ Year + (1|stk), data = df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
runDiagsME(mfit, "stk", df0, "diagMedI7.pdf", nc, nd)

# bootstrap
set.seed(1234)
stk <- unique(df0$stk)
mfit.bs <- split(1:it, 1:it) 
mfit.bs <- mclapply(mfit.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, stk==i))
	fit <- glmer(indF ~ Year + (1|stk), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)
# remove failed iters
mfit.bs <- mfit.bs[unlist(lapply(mfit.bs, is.numeric))]

mfitm <- do.call("rbind", mfit.bs)
mfitq <- apply(mfitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
mfitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(mfitq)))

# plot
png("figMedI7.png", 1800, 1200, res=300)
ggplot(mfitq, aes(x=Year)) + 
	geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) + 
	geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) + 
	geom_line(aes(y=`50%`)) + 
	expand_limits(y=0) + 
	geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) + 
	geom_hline(yintercept = 1, linetype=2) + 
	ylab(expression(F/F[MSY])) + 
	xlab("") + 
	theme(legend.position = "none") + 
	sc + 
	th
dev.off()

# table
tb0 <- t(mfitq)[-1,]
colnames(tb0) <- mfitq[,1]
write.csv(tb0, file="tabMedI7.csv")

#--------------------------------------------------------------------
# (I7b) F/Fmsy model regional
#--------------------------------------------------------------------
df0 <- sam
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))
# remove Eastern Med. where only 2 stocks are available 
df0 <- df0[df0$EcoRegion!="Eastern Med.",]

mfitRegional <- lapply(split(df0, df0$EcoRegion), function(x){
	# fit model
	mfit <- glmer(indF ~ Year + (1|stk), data = x, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
	# no variance with bootstrap due to small number of stocks
	mfit.pred <- predict(mfit, re.form=~0, type="response", newdata=nd)
	# output
	list(mfit=mfit, mfit.pred=mfit.pred)
})

# naming including No of stocks
No <- lapply(split(df0, df0$EcoRegion), function(x) length(unique(x$stk)))
names(mfitRegional) <- paste(names(No), " (", No, ")", sep="")

lst0 <- lapply(mfitRegional, "[[", "mfit.pred")
fIndfr <- data.frame(EcoRegion=rep(names(lst0), lapply(lst0, length)), N=unlist(lst0), Year=as.numeric(as.character(nd[,1])))

# plot
png("figMedI7b.png", 2400, 1200, res=300)
ggplot(fIndfr, aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	geom_hline(yintercept = 1, linetype=2) +
	ylab(expression(F/F[MSY])) + 
	xlab("") + 
	sc + 
	ylim(0, 3.5) + 
	th
dev.off()

# table
write.csv(dcast(fIndfr, EcoRegion~Year, value.var='N'), file="tabMedI7b.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I8) SSB indicator
#--------------------------------------------------------------------
# model
# pil_6 has a large impact in the indicator ...
idx <- !is.na(sam$SSB)
df0 <- sam[idx,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))
No <- length(unique(df0$stk))

# model
mfitb <- glmer(SSB ~ factor(Year) + (1|stk), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
runDiagsME(mfitb, "stk", df0, "diagMedI8.pdf", nc, nd)

# bootstrap
set.seed(1234)
stk <- unique(df0$stk)
mfitb.bs <- split(1:it, 1:it) 
mfitb.bs <- mclapply(mfitb.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, stk==i))
	fit <- glmer(SSB ~ Year + (1|stk), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)
# remove failed iters
mfitb.bs <- mfitb.bs[unlist(lapply(mfitb.bs, is.numeric))]

mfitm <- do.call("rbind", mfitb.bs)
mfitm <- exp(log(mfitm)-mean(log(mfitm[,1]), na.rm=TRUE))
mfitq <- apply(mfitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
mfitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(mfitq)))

# plot
png("figMedI8.png", 1800, 1200, res=300)
ggplot(mfitq, aes(x=Year)) + 
	geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) + 
	geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) + 
	geom_line(aes(y=`50%`)) + 
	expand_limits(y=0) + 
	geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) + 
	geom_hline(yintercept = 1, linetype=2) +
	ylab(expression(B/B[2003])) + 
	xlab("") + 
	theme(legend.position = "none") + 
	sc + 
	th
dev.off()

tb0 <- t(mfitq)[-1,]
colnames(tb0) <- mfitq[,1]
write.csv(tb0, file="tabMedI8.csv")

#--------------------------------------------------------------------
# (I8) SSB indicator regional
#--------------------------------------------------------------------
idx <- !is.na(sam$SSB)
df0 <- sam[idx,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))
# remove Eastern Med. where only 2 stocks are available 
df0 <- df0[df0$EcoRegion!="Eastern Med.",]

mfitbRegional <- lapply(split(df0, df0$EcoRegion), function(x){
	# fit model
	mfitb <- glmer(SSB ~ factor(Year) + (1|stk), data =  x, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
	# no variance with bootstrap due to small number of stocks
	mfitb.pred <- predict(mfitb, re.form=~0, type="response", newdata=nd)
	# output
	list(mfitb=mfitb, mfitb.pred=mfitb.pred/mfitb.pred[nd==iniYear])
})

# naming including No of stocks
No <- lapply(split(df0, df0$EcoRegion), function(x) length(unique(x$stk)))
names(mfitbRegional) <- paste(names(No), " (", No, ")", sep="")

lst0 <- lapply(mfitbRegional, "[[", "mfitb.pred")
bIndfr <- data.frame(EcoRegion=rep(names(lst0), lapply(lst0, length)), N=unlist(lst0), Year=as.numeric(as.character(nd[,1])))

# plot
png("figMedI8b.png", 2400, 1200, res=300)
ggplot(bIndfr, aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	geom_hline(yintercept = 1, linetype=2) +
	ylab(expression(B/B[2003])) + 
	xlab("") + 
	sc + 
	th
dev.off()

# table
write.csv(dcast(bIndfr, EcoRegion~Year, value.var='N'), file="tabMedI8b.csv", row.names=FALSE)

#--------------------------------------------------------------------

write.csv(sam, file="sam.csv")
save.image("RData.med")


