#####################################################################
# EJ(20190319)
# NEA indicators
#####################################################################

library(ggplot2)
library(lme4)
library(influence.ME)
library(lattice)
library(parallel)
library(rgdal)
library(reshape2)
library(plyr)
source("funs.R")

#====================================================================
# Setup
#====================================================================

# year when assessments were performed 
assessmentYear <- 2019
# final data year with estimations from stock assessments 
fnlYear <- assessmentYear - 1
# initial data year with estimations from stock assessments 
iniYear <- 2003
# vector of years
dy <- iniYear:fnlYear
# vector of years for valid assessments
vay <- (assessmentYear-2):assessmentYear
# vector of years for stock status projection
vpy <- (fnlYear-2):fnlYear
# options for reading data
options(stringsAsFactors=FALSE)
# number of simulations for mle bootstrap
it <- 500
# number of cores for mle bootstrap parallel
nc <- 1
# quantiles to be computed
qtl <- c(0.025, 0.25, 0.50, 0.75, 0.975)
# to control de seed in mclapply
RNGkind("L'Ecuyer-CMRG")
set.seed(1234)
# to make plots consistent
vp <- dy
vp[c(2,3,5,6,8,9,11,12,14,15)] <- ""
theme_set(theme_bw())
sc <- scale_x_continuous(breaks=dy, labels=as.character(vp))
th <- theme(axis.text.x  = element_text(angle=90, vjust=0.5), panel.grid.minor = element_blank())

#====================================================================
# load & pre-process
#====================================================================

#--------------------------------------------------------------------
# assessments
#--------------------------------------------------------------------
isa <- read.csv("../data/ices/Dataset_2020.csv", stringsAsFactors=FALSE)
#isa$FishingPressure <- as.numeric(isa$FishingPressure)

# extract the main ecoregion but keep the list
er <- strsplit(isa[,"EcoRegion"], ",")
isa$EcoRegionList <- isa$EcoRegion
isa$EcoRegion <- unlist(lapply(er, function(x) x[1]))
er <- strsplit(isa[,"EcoRegion"], " ")
isa$EcoRegion <- unlist(lapply(er, function(x) paste(x[-length(x)], collapse=" ")))
isa[isa$EcoRegion=="Bay of Biscay and the Iberian Coast", "EcoRegion"] <- "BoBiscay & Iberia"

# widely distributed to keep coherent with previous years (taken from 2017's files)
# >>> old codes don't exist anymore, updated to new ones
# isa[isa$OldFishStock %in% c("arg-rest", "bli-5b67", "boc-nea", "bsf-nea", "dgs-nea", "gfb-comb", "her-noss", "hke-nrtn", "hom-west", "lin-oth", "mac-nea", "rng-5b67", "smn-dp", "trk-nea", "usk-oth", "whb-comb"), "EcoRegion"] <- "Northeast Atlantic"
# This code was run in 2019 exercise 
# write.csv(unique(isa[isa$OldFishStock %in% c("arg-rest", "bli-5b67", "boc-nea", "bsf-nea", "dgs-nea", "gfb-comb", "her-noss", "hke-nrtn", "hom-west", "lin-oth", "mac-nea", "rng-5b67", "smn-dp", "trk-nea", "usk-oth", "whb-comb"),c("OldFishStock", "FishStock")]), file="widelyDistributed.csv")
isa[isa$FishStock %in% c("dgs.27.nea", "aru.27.6b7-1012", "bli.27.5b67", "hke.27.3a46-8abd", "mac.27.nea", "whb.27.1-91214", "hom.27.2a4a5b6a7a-ce-k8", "reb.2127.dp", "lin.27.3a4a6-91214", "usk.27.3a45b6a7-912b", "rng.27.5b6712b", "bsf.27.nea", "her.27.1-24a514a", "boc.27.6-8", "sdv.27.nea", "gfb.27.nea"), "EcoRegion"] <- "Northeast Atlantic"
             
# a couple of stocks that need fixing
# correcting Greater North Sea
isa[isa$FishStock %in% c("had.27.46a20", "pok.27.3a46", "sol.27.7e"), "EcoRegion"] <- "Greater North Sea"

# fix codes for stock size and fishing mortality
isa[isa$FishingPressureDescription %in% c("Harvest Rate", "Harvest rate"), "FishingPressureDescription"] <- "HR"

# order by year
isa <- isa[order(isa$Year),]

# reporting stk by data category
stBydc <- unique(subset(isa, Year %in% vpy)[,c("FishStock", "DataCategory", "EcoRegion")])
stBydc <- transform(stBydc, cat=as.integer(DataCategory))
write.csv(table(stBydc[,c("EcoRegion","cat")]), file="stBydc.csv")

#--------------------------------------------------------------------
# ICES rectangles data
#--------------------------------------------------------------------

rectangles <- readOGR("../data/ices_areas", layer= "ICES_StatRec_map_Areas_Full_20170124")
rectangles <- rectangles@data[,c("Area_27", "AreasList", "ICESNAME")]
colnames(rectangles) <- c("Max_Area","Area_List", "Rectangle")
rectangles <- subset(rectangles, !is.na(Max_Area))
# A new column is added based on Max_Area so that it is comparable across the other data sets
rectangles$Area <- paste("27.",toupper(as.character(rectangles$Max_Area)),sep="")
# Check that each rectangle is unique and only appears once in the data
# i.e. each rectangle is uniquely assigned to one area
length(unique(rectangles$Rectangle)) == nrow(rectangles)

#--------------------------------------------------------------------
# sampling frame (TACs)
#--------------------------------------------------------------------

load("../data/ices/sframe.RData")
# fmz is the frame of all TACs
# For consistency
colnames(fmz)[colnames(fmz) == "area"] <- "Area"
colnames(fmz)[colnames(fmz) == "spp"] <- "Species"
colnames(fmz)[colnames(fmz) == "stock_id"] <- "TAC_id"
sframe <- subset(fmz, TAC_id %in% sframe_TAC)

# Each ICES area should only appear once for each FMZ stock (to prevent the appearance of duplicate rectangles when merging with the ICES rectangle data later). We check this here:

unarea <- daply(sframe, .(TAC_id), function(x){
    return(length(unique(x$Area))==nrow(x))
})
all(unarea)

#====================================================================
# Stocks to retain
# matches sampling frame and ICES assessments through ICES rectangles
#====================================================================

#--------------------------------------------------------------------
# subset assessments and ecoregions, add areas
#--------------------------------------------------------------------

# remove 3+
cols <- c("FishStock","ICES.Areas..splited.with.character....." , "SpeciesName", "SGName", "DataCategory", "EcoRegion")
isa12 <- isa[isa$DataCategory<3, cols]

# NOTE: should do these fixes to isa and after subset to isa12
colnames(isa12)[colnames(isa12) == "ICES.Areas..splited.with.character....."] <- "Areas"
# Drop duplicates
isa12 <- unique(isa12)
# Remove white space and any capital letters from assessment name
isa12[,"FishStock"] <- tolower(gsub("\\s", "", isa12[,"FishStock"]))
# Make a species column from the assessment name
spp <- strsplit(isa12[,"FishStock"], "\\.")
isa12$Species <- toupper(unlist(lapply(spp, function(x) x[1])))
# Split ICES area by ~
areas <- strsplit(isa12[,"Areas"], "~")
names(areas) <- isa12[,"FishStock"]
areas <- melt(areas)
colnames(areas) <- c("Area","FishStock")
isa12 <- merge(isa12, areas)
# keep relevant columns only
isa12 <- isa12[,c("FishStock","Area", "Species", "SpeciesName", "SGName", "DataCategory", "EcoRegion")]
isa12[,"Area"] <- toupper(gsub("\\s", "", isa12[,"Area"]))
# remove ecoregions outside EU waters
isa12 <- subset(isa12, !(EcoRegion %in% c("Arctic Ocean", "Greenland Sea", "Faroes", "Iceland Sea")))
# drop if ecoregion is NA
isa12 <- subset(isa12, !is.na(EcoRegion))
# remove her-noss which is widely distributed but mainly norway
isa12 <- subset(isa12, FishStock!="her.27.1-24a514a")

#--------------------------------------------------------------------
# fix area codes
#--------------------------------------------------------------------

# fix Baltic area codes
rectangles[rectangles$Area == "27.3.A.20","Area"] <- "27.3.A"
rectangles[rectangles$Area == "27.3.A.21","Area"] <- "27.3.A"
rectangles[rectangles$Area == "27.3.B.23","Area"] <- "27.3.B"
rectangles[rectangles$Area == "27.3.C.22","Area"] <- "27.3.C"

isa12[isa12$Area == "27.3.A.20","Area"] <- "27.3.A"
isa12[isa12$Area == "27.3.A.21","Area"] <- "27.3.A"
isa12[isa12$Area == "27.3.B.23","Area"] <- "27.3.B"
isa12[isa12$Area == "27.3.C.22","Area"] <- "27.3.C"

sframe[sframe$Area == "27.3.20","Area"] <- "27.3.A"
sframe[sframe$Area == "27.3.21","Area"] <- "27.3.A"
sframe[sframe$Area == "27.3.23","Area"] <- "27.3.B"
sframe[sframe$Area == "27.3.22","Area"] <- "27.3.C"

# Check: shouldn't have any 24.x.x areas
# Areas in ICES assessment but missing in rectangles
### rewrite
unique(isa12$Area)[!(unique(isa12$Area) %in% unique(rectangles$Area))]
#[1] "21.1" "21.2"

# Areas in FMZ but missing in rectangles
unique(sframe$Area)[!(unique(sframe$Area) %in% unique(rectangles$Area))]
#[1] "21.1.F"  "21.3.M"  "34.1.2"  "34.1.13" "34.1.11" "34.1.12" "34.2" 

#--------------------------------------------------------------------
# fix species codes
#--------------------------------------------------------------------
#check the species code 
# Horse mackerel
# Checked in 2019 and HOM still exists
isa12[isa12$Species=="HOM","Species"] <- "JAX"
# ANK & MON - Anglerfish - species to genus
# Checked in 2019 and ANK+MON still exist
isa12[isa12$Species=="ANK","Species"] <- "ANF"
isa12[isa12$Species=="MON","Species"] <- "ANF"
# Megrim - species and genus to genus
# Checked in 2019 and MEG+LDB still exist
isa12[isa12$Species=="MEG","Species"] <- "LEZ"
isa12[isa12$Species=="LDB","Species"] <- "LEZ"
# species with combined TACs (NOTE THESE CAN INCREASE IN THE FUTURE)
# WIT there's a combined TAC with lemon sole: L/W/2AC4-C
# TUR there's a combined TAC with brill T/B/2AC4-C
# Both TUR and WIT were not cat 1 in 2017 assessments
isa12[isa12$Species=="WIT","Species"] <- "L/W"
isa12[isa12$Species=="TUR","Species"] <- "T/B"
# missing species
sort(unique(isa12$Species)[!(unique(isa12$Species) %in% unique(sframe$Species))])
#[1] "BSS" "PIL" "REB"
# REB is in areas outside EU waters 27.5, 27.12, 27.14
# PIL and BSS don't have TACs

#--------------------------------------------------------------------
# merge assessments,tacs/sf and rectangles
#--------------------------------------------------------------------

# merge assessments with rectangles
isa12r <- merge(isa12, rectangles[,c("Area","Rectangle")], by="Area")

# Do we have all the assessments?
all(sort(unique(isa12$FishStock)) == sort(unique(isa12r$FishStock)))

# Merge sampling frame with rectangles
sfr <- merge(sframe, rectangles[,c("Area","Rectangle")], by="Area")

# Do we have all the TACs?
all(sort(unique(sframe$TAC_id)) == sort(unique(sfr$TAC_id)))

# merge assessments with sampling frame
isa12sf <- merge(sfr, isa12r[,c("Species","Rectangle","FishStock","DataCategory")], by=c("Species","Rectangle"), all.x = TRUE)

#--------------------------------------------------------------------
# final stock list
#--------------------------------------------------------------------

# remove stocks with short time series
sts <- subset(isa, Year %in% dy & !is.na(FishingPressure))$FishStock
# remove short time series
sts <- table(sts)
sts <- names(sts)[sts<5]
# remove also nep.fu.3-4, assessment area is not stable so doesn't have 5 years of comparable data 
sts <- c(sts, "nep.fu.3-4")

# stocks to retain
stkToRetain <- unique(isa12sf$FishStock)[-1]
stkToRetain <- stkToRetain[!(stkToRetain %in% sts)]

#--------------------------------------------------------------------
# subset assessments
#--------------------------------------------------------------------
# filtering
saeu <- subset(isa, FishStock %in% stkToRetain)

# reporting
stkToDrop <- unique(isa[!(isa$FishStock %in% stkToRetain), c("FishStock", "EcoRegion", "DataCategory")])
write.csv(stkToDrop, file="stkToDropBySampFrame-nea.csv")
stkToRetain <- unique(isa[isa$FishStock %in% stkToRetain, c("FishStock", "EcoRegion", "DataCategory")])
write.csv(stkToRetain, file="stkToRetainBySampFrame-nea.csv")

# check what's available
table(saeu[,c("FishingPressureDescription","StockSizeDescription")])

#====================================================================
# process data for indicators
#====================================================================

#--------------------------------------------------------------------
# fixing BMSYescapment not reported by ICES
#--------------------------------------------------------------------
saeu$MSYBescapement <- NA

# NOP 34 
saeu[saeu$FishStock == "nop.27.3a4", c("StockSize", "MSYBescapement")] <- saeu[saeu$FishStock == "nop.27.3a4", c("Low_StockSize", "Blim")]

# ANE BISC - need to add value from ss, using upper trigger as proxy for MSYBescapement
saeu[saeu$FishStock == "ane.27.8", "MSYBescapement"] <- 89000 

# acording to the sumsheets SAN and SPR-NSEA use Bpa for MSYBescapement
saeu[saeu$FishStock %in% c("san.sa.1r","san.sa.2r","san.sa.3r","san.sa.4","spr.27.3a4"),"MSYBescapement"] <- saeu[saeu$FishStock %in% c("san.sa.1r","san.sa.2r","san.sa.3r","san.sa.4","spr.27.3a4"),"Bpa"]

#--------------------------------------------------------------------
# fixing Recruitments of 0
#--------------------------------------------------------------------
saeu[saeu$Recruitment==0 & !is.na(saeu$Recruitment),"Recruitment"] <- NA

#--------------------------------------------------------------------
# Bref
#--------------------------------------------------------------------
# check MSYBtrigger approx. Bpa, need some boundaries for rounding 
stksBpaMSYBtrigger <- unique(saeu[saeu$MSYBtrigger/saeu$Bpa < 1.05 & saeu$MSYBtrigger/saeu$Bpa > 0.95, c("FishStock", "Bpa", "MSYBtrigger")])
stksBpaMSYBtrigger <- stksBpaMSYBtrigger[order(stksBpaMSYBtrigger$FishStock),]
write.csv(stksBpaMSYBtrigger, file="stksBpaMSYBtrigger.csv")

# create field
saeu$Bref <- saeu$MSYBtrigger
# if MSYBtrigger is set at Bpa level set to NA, with the exception 
# of a couple of stocks which were explicitly set that way by the AWG
saeu$Bref[saeu$MSYBtrigger==saeu$Bpa & !(saeu$FishStock %in% c("hom.27.9a", "pra.27.3a4a", "sol.27.7e"))] <- NA

# B escapement as Bref for relevant stocks
saeu$Bref[!is.na(saeu$MSYBescapement)] <- saeu$MSYBescapement[!is.na(saeu$MSYBescapement)]
saeu$Bref <- as.numeric(saeu$Bref)
# set 0 as NA
saeu$Bref[saeu$Bref==0] <- NA
# if relative Bref = 1
saeu[saeu$StockSizeDescription == "B/Bmsy", "Bref"] <- 1

# Bpa
saeu$Brefpa <- saeu$Bpa
# some stocks don't have Bpa (it was set at MSYBtrigger level)
saeu$Brefpa[saeu$FishStock %in% c("hom.27.9a")] <- NA
# set 0 as NA
saeu$Brefpa[saeu$Brefpa==0] <- NA
# if relative Brefpa = 0.5
saeu[saeu$StockSizeDescription == "B/Bmsy", "Brefpa"] <- 0.5

#--------------------------------------------------------------------
# Fref
#--------------------------------------------------------------------
saeu$Fref <- saeu$FMSY
# no Fref for B escapement 
saeu$Fref[!is.na(saeu$MSYBescapement)] <- NA
saeu$Fref <- as.numeric(saeu$Fref)
# set 0 as NA
saeu$Fref[saeu$Fref==0] <- NA
# if relative Fmsy must be 1
saeu[saeu$FishingPressureDescription %in% c("F/Fmsy", "HR/HRmsy"), "Fref"] <- 1

saeu$Frefpa <- saeu$Fpa
# no Fref for B escapement 
saeu$Frefpa[!is.na(saeu$MSYBescapement)] <- NA
saeu$Frefpa <- as.numeric(saeu$Frefpa)
# set 0 as NA
saeu$Frefpa[saeu$Frefpa==0] <- NA
# if relative Fparef must be NA
saeu[saeu$FishingPressureDescription %in% c("F/Fmsy", "HR/HRmsy"), "Frefpa"] <- NA

#--------------------------------------------------------------------
# COMPUTE F/Fref and B/Bref | year + stock
#--------------------------------------------------------------------
saeu <- transform(saeu, 
	indF = FishingPressure/Fref, 
	indB=StockSize/Bref, 
	indBpa=StockSize/Brefpa, 
	indFpa = FishingPressure/Frefpa)

# in case of escapement strategy MSY evaluated by SSB ~ Bref
saeu$indF[!is.na(saeu$MSYBescapement)] <- 
	saeu$Bref[!is.na(saeu$MSYBescapement)]/saeu$StockSize[!is.na(saeu$MSYBescapement)]

saeu <- transform(saeu, sfFind=!is.na(indF))

#--------------------------------------------------------------------
# COMPUTE SBL | year + FishStock
#--------------------------------------------------------------------
saeu$SBL <- !(saeu$indFpa > 1 | saeu$indBpa < 1)
# if one is NA SBL can't be inferred
saeu$SBL[is.na(saeu$indFpa) | is.na(saeu$indBpa)] <- NA
# no SBL for B escapement 
saeu$SBL[!is.na(saeu$MSYBescapement)] <- NA
saeu <- transform(saeu, sfSBL=!is.na(SBL))

#--------------------------------------------------------------------
# COMPUTE CFP objectives | year + FishStock
#--------------------------------------------------------------------
saeu$CFP <- !(saeu$indF > 1 | saeu$indB < 1)
# if one is NA CFP can't be inferred
saeu$CFP[is.na(saeu$indF) | is.na(saeu$indB)] <- NA
# no CFP for B escapement 
saeu$CFP[!is.na(saeu$MSYBescapement)] <- NA
saeu <- transform(saeu, sfCFP=!is.na(CFP))

#--------------------------------------------------------------------
# final dataset
#--------------------------------------------------------------------
# remove WG projections
saeu <- saeu0 <- subset(saeu, Year <= fnlYear)
saeu <- subset(saeu, Year>=iniYear & assessmentYear %in% vay & sfFind)

#--------------------------------------------------------------------
# project stock status up to last year in cases missing
#--------------------------------------------------------------------

saeu <- projectStkStatus(saeu, vpy)

moo1 <- saeu[!saeu$projected, c("FishStock", "Year", "EcoRegion")]
moo2 <- table(moo1[,c("FishStock", "Year", "EcoRegion")])
moo2 <- dcast(data.frame(moo2), FishStock~Year, value.var = 'Freq' )
#====================================================================
# Indicators (design based)
#====================================================================

#--------------------------------------------------------------------
# Number of stocks (remove projected years)
#--------------------------------------------------------------------
df0 <- saeu[!saeu$projected,]
inStks <- getNoStks(df0, "FishStock", length)

# check for potential duplicates
mo1 <- df0[df0$EcoRegion == "Greater North Sea", c("EcoRegion", "FishStock", "Year")]
table(mo1[,c("FishStock", "Year")])

png("figNEAI0a.png", 1800, 1200, res=300)
ggplot(subset(inStks, EcoRegion=="ALL"), aes(x=Year, y=N)) + 
	geom_line() + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,80)) + 
	sc + 
	th  
dev.off()

# time series
# check stocks with non continuos time series
# plot needs to be fixed mannualy but should be possible to auto
stks_ncts <- tapply(df0$Year, df0$FishStock, function(x){
	!(max(x) - min(x) + 1 == length(x))
})
stks_ncts <- names(stks_ncts)[stks_ncts]

png("figNEAI0b.png", 3000, 4500, res=300, bg = "transparent")
ggplot(df0, aes(Year, reorder(FishStock, desc(FishStock)))) + 
	geom_line() + 
	geom_point(data=aggregate(list(Year=df0$Year, EcoRegion=df0$EcoRegion),
	    by=list(FishStock=df0$FishStock), max)) +
  # NEP missing years
	geom_line(data=data.frame(Year=2009:2013, FishStock="nep.fu.14",
    EcoRegion="Celtic Seas"), color="white") +
	geom_line(data=data.frame(Year=2007:2009, FishStock="nep.fu.13",
    EcoRegion="Celtic Seas"), color="white") +
	geom_line(data=data.frame(Year=2003:2005, FishStock="nep.fu.13",
    EcoRegion="Celtic Seas"), color="white") +
	geom_point(data=data.frame(Year=2003, FishStock="nep.fu.13",
    EcoRegion="Celtic Seas"), size=0.3) + 
	ylab("") +
	xlab("Year") +
	sc + 
	th +
	facet_grid(EcoRegion~., switch="y", space="free_y", scales="free_y") +
	theme(strip.placement="outside", strip.background.y=element_blank(),
    panel.spacing.y=unit(0.05, "lines"))
dev.off()

write.csv(dcast(inStks, EcoRegion~Year, value.var='N'), file="tabNEAI0.csv", row.names=FALSE) 

#--------------------------------------------------------------------
# (I1) Stocks F > Fmsy 
#--------------------------------------------------------------------
fInda <- getNoStks(saeu, "indF", function(x) sum(x>1))

# plot
png("figNEAI1.png", 1800, 1200, res=300)
ggplot(subset(fInda, EcoRegion=='ALL'), aes(x=Year, y=N)) + 
	geom_line() +
	expand_limits(y=0) + 
	geom_point(aes(x=iniYear, y=N[1])) +
	geom_point(aes(x=fnlYear, y=N[length(N)]), size=2) + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,75)) + 
	sc + 
	th 
dev.off()

# plot
png("figNEAI1b.png", 2400, 1200, res=300)
ggplot(subset(fInda, EcoRegion != 'ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab("No. of stocks") + 
	xlab("") + 
	sc + 
	ylim(0, 20) + 
	th
dev.off()

# table
write.csv(dcast(fInda, EcoRegion~Year, value.var='N'), file="tabNEAI1.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I2) Stocks F <= Fmsy
#--------------------------------------------------------------------
fIndb <- getNoStks(saeu, "indF", function(x) sum(x<=1))

# plot
png("figNEAI2.png", 1800, 1200, res=300)
ggplot(subset(fIndb, EcoRegion=='ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	expand_limits(y=0) + 
	geom_point(aes(x=iniYear, y=N[1])) + 
	geom_point(aes(x=fnlYear, y=N[length(N)]), size=2) +
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,75)) + 
	sc + 
	th
dev.off()

# plot
png("figNEAI2b.png", 2400, 1200, res=300)
ggplot(subset(fIndb, EcoRegion != 'ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab("No. of stocks") + 
	xlab("") + 
	sc + 
	ylim(0, 20) + 
	th
dev.off()

# table
write.csv(dcast(fIndb, EcoRegion~Year, value.var='N'), file="tabNEAI2.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I3) Stocks outside SBL
#--------------------------------------------------------------------
fIndc <- getNoStks(saeu, "SBL", function(x) sum(!x, na.rm=TRUE))

# plot
png("figNEAI3.png", 1800, 1200, res=300)
ggplot(subset(fIndc, EcoRegion=='ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	expand_limits(y=0) + 
	geom_point(aes(x=iniYear, y=N[1])) +
	geom_point(aes(x=fnlYear, y=N[length(N)]), size=2) + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,75)) + 
	sc + 
	th
dev.off()

# plot
png("figNEAI3b.png", 2400, 1200, res=300)
ggplot(subset(fIndc, EcoRegion != 'ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab("No. of stocks") + 
	xlab("") + 
	sc + 
	ylim(0, 15) + 
	th
dev.off()

# table
write.csv(dcast(fIndc, EcoRegion~Year, value.var='N'), file="tabNEAI3.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I4) Stocks inside SBL
#--------------------------------------------------------------------
fIndd <- getNoStks(saeu, "SBL", function(x) sum(x, na.rm=TRUE))

## plot
png("figNEAI4.png", 1800, 1200, res=300)
ggplot(subset(fIndd, EcoRegion=='ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	expand_limits(y=0) + 
	geom_point(aes(x=iniYear, y=N[1])) +
	geom_point(aes(x=fnlYear, y=N[length(N)]), size=2) + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,75)) + 
	sc + 
	th
dev.off()

# plot
png("figNEAI4b.png", 2400, 1200, res=300)
ggplot(subset(fIndd, EcoRegion != 'ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab("No. of stocks") + 
	xlab("") + 
	sc + 
	ylim(0, 15) + 
	th
dev.off()

# table
write.csv(dcast(fIndd, EcoRegion~Year, value.var='N'), file="tabNEAI4.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I5) Stocks outside CFP objectives
#--------------------------------------------------------------------
fIndf <- getNoStks(saeu, "CFP", function(x) sum(!x, na.rm=TRUE))

## plot
png("figNEAI5.png", 1800, 1200, res=300)
ggplot(subset(fIndf, EcoRegion=='ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	expand_limits(y=0) + 
	geom_point(aes(x=iniYear, y=N[1])) +
	geom_point(aes(x=fnlYear, y=N[length(N)]), size=2) + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,75)) + 
	sc + 
	th
dev.off()

# plot
png("figNEAI5b.png", 2400, 1200, res=300)
ggplot(subset(fIndf, EcoRegion != 'ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab("No. of stocks") + 
	xlab("") + 
	sc + 
	ylim(0, 20) + 
	th
dev.off()

# table
write.csv(dcast(fIndf, EcoRegion~Year, value.var='N'), file="tabNEAI5.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I6) Stocks inside CFP objectives
#--------------------------------------------------------------------
fIndfb <- getNoStks(saeu, "CFP", function(x) sum(x, na.rm=TRUE))

# plot
png("figNEAI6.png", 1800, 1200, res=300)
ggplot(subset(fIndfb, EcoRegion=='ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	expand_limits(y=0) + 
	geom_point(aes(x=iniYear, y=N[1])) +
	geom_point(aes(x=fnlYear, y=N[length(N)]), size=2) + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,75)) + 
	sc + 
	th
dev.off()

# plot
png("figNEAI6b.png", 2400, 1200, res=300)
ggplot(subset(fIndfb, EcoRegion != 'ALL'), aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab("No. of stocks") + 
	xlab("") + 
	sc + 
	ylim(0, 20) + 
	th
dev.off()

# table
write.csv(dcast(fIndfb, EcoRegion~Year, value.var='N'), file="tabNEAI6.csv", row.names=FALSE)

#====================================================================
# Indicators (model based)
#====================================================================

#--------------------------------------------------------------------
# (I7) F/Fmsy model
#--------------------------------------------------------------------
idx <- saeu$FishingPressureDescription %in% c("F", "F/Fmsy")
saeu$sfI7 <- idx & is.na(saeu$MSYBescapement)
df0 <- saeu[saeu$sfI7,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

# fit
ifit <- glmer(indF ~ Year + (1|FishStock), data = df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
runDiagsME(ifit, "FishStock", df0, "diagNEAI7.pdf", nc, nd)

# bootstrap
stk <- unique(df0$FishStock)
ifit.bs <- split(1:it, 1:it) 

ifit.bs <- mclapply(ifit.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(indF ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifit.bs)
ifitq <- apply(ifitm, 2, quantile, qtl, na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAI7.png", 1800, 1200, res=300)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  geom_hline(yintercept = 1, linetype=2) +
  ylab(expression(F/F[MSY])) + 
  ylim(0, 2.5) + 
  xlab("") +
  theme(legend.position = "none") + 
  sc + 
  th
dev.off()

# table
tb0 <- t(ifitq)[-1,]
colnames(tb0) <- ifitq[,1]
write.csv(tb0, file="tabNEAI7.csv")

#--------------------------------------------------------------------
# (I7b) F/Fmsy model regional
#--------------------------------------------------------------------
df0 <- saeu[saeu$sfI7,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

ifitRegional <- lapply(split(df0, df0$EcoRegion), function(x){
	# fit model
	ifit <- glmer(indF ~ Year + (1|FishStock), data = x, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
	# no variance with bootstrap due to small number of stocks
	ifit.pred <- predict(ifit, re.form=~0, type="response", newdata=nd)
	# output
	list(ifit=ifit, ifit.pred=ifit.pred)
})

# naming including No of stocks
No <- lapply(split(df0, df0$EcoRegion), function(x) length(unique(x$FishStock)))
names(ifitRegional) <- paste(names(No), " (", No, ")", sep="")

lst0 <- lapply(ifitRegional, "[[", "ifit.pred")
fIndfr <- data.frame(EcoRegion=rep(names(lst0), lapply(lst0, length)), N=unlist(lst0), Year=as.numeric(as.character(nd[,1])))

# plot
png("figNEAI7b.png", 2400, 1200, res=300)
ggplot(fIndfr, aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	ylab(expression(F/F[MSY])) + 
	xlab("") + 
	sc + 
	ylim(0, 2.5) + 
	th
dev.off()

# table
write.csv(dcast(fIndfr, EcoRegion~Year, value.var='N'), file="tabNEAI7b.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I7out) F/Fmsy stocks outside EU
#--------------------------------------------------------------------
df0 <- subset(isa, (EcoRegion %in% c("Arctic Ocean", "Greenland Sea", "Faroes", "Iceland Sea") | FishStock=="her.27.1-24a514a") & FishStock!="pra.27.1-2" & Year>=iniYear & Year<=fnlYear & AssessmentYear %in% vay)
df0$Fref <- as.numeric(df0$FMSY)
df0 <- transform(df0, indF = FishingPressure/Fref, sfFind=!is.na(FishingPressure/Fref))
idx <- df0$FishingPressureDescription %in% c("F", "F/Fmsy") & df0$sfFind
df0 <- df0[idx,]

# check data series is complete
table(df0[,c("FishStock","Year")])

# create year variable for prediction
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

# fit
ifitout <- glmer(indF ~ Year + (1|FishStock), data = df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
runDiagsME(ifitout, "FishStock", df0, "diagNEAI7out.pdf", nc, nd)

# bootstrap
stk <- unique(df0$FishStock)
ifitout.bs <- split(1:it, 1:it) 
ifitout.bs <- mclapply(ifitout.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(indF ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifitout.bs)
ifitq <- apply(ifitm, 2, quantile, qtl, na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAI7out.png", 1800, 1200, res=300)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  ylab(expression(F/F[MSY])) + 
  geom_hline(yintercept = 1, linetype=2) +
  ylim(0, 2.5) + 
  xlab("") +
  theme(legend.position = "none") + 
  sc + 
  th
dev.off()

# table
tb0 <- t(ifitq)[-1,]
colnames(tb0) <- ifitq[,1]
write.csv(tb0, file="tabNEAI7out.csv")

#--------------------------------------------------------------------
# (I8) SSB model
#--------------------------------------------------------------------
saeu$sfI8 <- saeu$StockSizeDescription %in% c("SSB", "TSB")
df0 <- saeu[saeu$sfI8,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

# fit
ifitb <- glmer(StockSize ~ Year + (1|FishStock), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
runDiagsME(ifitb, "FishStock", df0, "diagNEAI8.pdf", nc, nd)

# bootstrap
stk <- unique(df0$FishStock)
ifitb.bs <- split(1:it, 1:it) 
ifitb.bs <- mclapply(ifitb.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(StockSize ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifitb.bs)
ifitm <- exp(log(ifitm)-median(log(ifitm[,1]), na.rm=TRUE))
ifitq <- apply(ifitm, 2, quantile, qtl, na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAI8.png", 1800, 1200, res=300)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + 
  expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  geom_hline(yintercept = 1, linetype=2) +
  ylab(expression(B/B[2003])) + 
  xlab("") +
  theme(legend.position = "none") + 
  sc + 
  th
dev.off()

# table
tb0 <- t(ifitq)[-1,]
colnames(tb0) <- ifitq[,1]
write.csv(tb0, file="tabNEAI8.csv")

#--------------------------------------------------------------------
# (I8b) SSB model regional
#--------------------------------------------------------------------
df0 <- saeu[saeu$sfI8,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

ifitbRegional <- lapply(split(df0, df0$EcoRegion), function(x){
	# fit model
	ifitb <- glmer(StockSize ~ Year + (1|FishStock), data = x, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
	# no variance with bootstrap due to small number of stocks
	ifitb.pred <- predict(ifitb, re.form=~0, type="response", newdata=nd)
	# output
	list(ifitb=ifitb, ifitb.pred=ifitb.pred/ifitb.pred[nd==iniYear])
})

# naming including No of stocks
No <- lapply(split(df0, df0$EcoRegion), function(x) length(unique(x$FishStock)))
names(ifitbRegional) <- paste(names(No), " (", No, ")", sep="")

lst0 <- lapply(ifitbRegional, "[[", "ifitb.pred")
fIndbr <- data.frame(EcoRegion=rep(names(lst0), lapply(lst0, length)), N=unlist(lst0), Year=as.numeric(as.character(nd[,1])))

# plot
png("figNEAI8b.png", 2400, 1200, res=300)
ggplot(fIndbr, aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	geom_hline(yintercept = 1, linetype=2) +
	ylab(expression(B/B[2003])) + 
	xlab("") +
	theme(legend.position = "none") + 
	sc + 
	th
dev.off()

# table
write.csv(dcast(fIndbr, EcoRegion~Year, value.var='N'), file="tabNEAI8b.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I10) Recruitment model
#--------------------------------------------------------------------
saeu0 <- saeu0[!saeu0$FishStock %in% "cod.27.24-32",] # excluded from the analysis due to the absence of F reference points

saeu0$sfI10 <- !is.na(saeu0$Recruitment)
df0 <- saeu0[saeu0$sfI10,]
# data for table about stocks and inicators 
sfI10 <- subset(df0, Year>=iniYear & Year<=fnlYear)
sfI10 <- tapply(sfI10$Year, sfI10$FishStock, max)
sfI10 <- data.frame(FishStock=names(sfI10), Year=sfI10, variable="sfI10", value=TRUE)
# project and compute indicator
df0 <- projectStkStatus(df0, vpy)
for(i in (iniYear):fnlYear) df0 <- decadalR(df0, i)
df0 <- subset(df0, Year>=iniYear & Year<=fnlYear)
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

# fit
ifitr <- glmer(decadalR ~ Year + (1|FishStock), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
runDiagsME(ifitr, "FishStock", df0, "diagNEAI10.pdf", nc, nd)

# bootstrap
stk <- unique(df0$FishStock)
ifitr.bs <- split(1:it, 1:it) 
ifitr.bs <- mclapply(ifitr.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(decadalR ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifitr.bs)
ifitm <- exp(log(ifitm)-median(log(ifitm[,1]), na.rm=TRUE))
ifitq <- apply(ifitm, 2, quantile, qtl, na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

#load("./RData_nea_newrec")
# plot
png("figNEAI10.png", 1800, 1200, res=300)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + 
  expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  geom_hline(yintercept = 1, linetype=2) +
  #ylab(expression(decadal_R/R[2003])) + 
  ylab("Decadal recruitment (scaled to 2003)") + 
  xlab("") +
  theme(legend.position = "none") + 
  sc + 
  th
dev.off()

# table
tb0 <- t(ifitq)[-1,]
colnames(tb0) <- ifitq[,1]
write.csv(tb0, file="tabNEAI10.csv")

#--------------------------------------------------------------------
# (I10b) R model regional
#--------------------------------------------------------------------

ifitrRegional <- lapply(split(df0, df0$EcoRegion), function(x){
	# fit model
	ifitr <- glmer(decadalR ~ Year + (1|FishStock), data = x, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
	# no variance with bootstrap due to small number of stocks
	ifitr.pred <- predict(ifitr, re.form=~0, type="response", newdata=nd)
	# output
	list(ifitr=ifitr, ifitr.pred=ifitr.pred/ifitr.pred[nd==iniYear])
})

# naming including No of stocks
No <- lapply(split(df0, df0$EcoRegion), function(x) length(unique(x$FishStock)))
names(ifitrRegional) <- paste(names(No), " (", No, ")", sep="")

lst0 <- lapply(ifitrRegional, "[[", "ifitr.pred")
fIndrr <- data.frame(EcoRegion=rep(names(lst0), lapply(lst0, length)), N=unlist(lst0), Year=as.numeric(as.character(nd[,1])))

# plot
png("figNEAI10b.png", 2400, 1200, res=300)
ggplot(fIndrr, aes(x=Year, y=N)) + 
	geom_line() + 
	facet_grid(.~EcoRegion) +
	geom_hline(yintercept = 1, linetype=2) +
    ylab("Decadal recruitment (scaled to 2003)") + 
	xlab("") +
	theme(legend.position = "none") + 
	sc + 
	th
dev.off()

# table
write.csv(dcast(fIndrr, EcoRegion~Year, value.var='N'), file="tabNEAI10b.csv", row.names=FALSE)

#--------------------------------------------------------------------
# (I12) SSB model for cat 3
# >>> Check which are in sampling frame
# >>> Add to report Abundance indices also used
#--------------------------------------------------------------------

df0 <- subset(isa, !(EcoRegion %in% c("Arctic Ocean", "Greenland Sea", "Faroes", "Iceland Sea")) & DataCategory>2 & DataCategory<4 & StockSize>0 &  Year>=iniYear & Year <= fnlYear & AssessmentYear %in% vay & StockSizeDescription %in% c("Biomass index", "Abundance index", "SSB", "TSB", "Relative BI (comb)", "B/Bmsy", "Relative SSB", "standardized CPUE", "Relative BI", "Biomass Index (comb)", "LPUE"))

# remove stocks with short time series
sts <- table(df0$FishStock, df0$Year)
sts <- rownames(sts)[apply(sts, 1, sum)<5]
df0 <- subset(df0, !(FishStock %in% sts))

# id
sfI12 <- tapply(df0$Year, df0$FishStock, max)
sfI12 <- data.frame(FishStock=names(sfI12), Year=sfI12, variable="sfI12", value=TRUE)

# project for stocks without 2015, 2016 estimates
# NEED CHECK
df0 <- projectStkStatus(df0, vpy)

# pre process for model 
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

# fit
ifitb3 <- glmer(StockSize ~ Year + (1|FishStock), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
runDiagsME(ifitb3, "FishStock", df0, "diagNEAI12.pdf", nc, nd)

# bootstrap
stk <- unique(df0$FishStock)
ifitb3.bs <- split(1:it, 1:it) 
ifitb3.bs <- mclapply(ifitb3.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(StockSize ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifitb3.bs)
ifitm <- exp(log(ifitm)-median(log(ifitm[,1]), na.rm=TRUE))
ifitq <- apply(ifitm, 2, quantile, qtl, na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAI12.png", 1800, 1200, res=300)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + 
  expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  geom_hline(yintercept = 1, linetype=2) +
  ylab(expression(B/B[2003])) + 
  xlab("") +
  theme(legend.position = "none") + 
  sc + 
  th
dev.off()

tb0 <- t(ifitq)[-1,]
colnames(tb0) <- ifitq[,1]
write.csv(tb0, file="tabNEAI12.csv")

#====================================================================
# Bootstrap convergence problems
#====================================================================

bootconv <- data.frame(
	indicator=c('F/Fmsy trends', 'F/Fmsy trends out', 'Biomass trends', 'Decadal recruitment trends', "Biomass data category 3 trends"),
	convergence=c(sum(unlist(lapply(lapply(ifit.bs, is.na), sum))==0), sum(unlist(lapply(lapply(ifitout.bs, is.na), sum))==0), sum(unlist(lapply(lapply(ifitb.bs, is.na), sum))==0), sum(unlist(lapply(lapply(ifitr.bs, is.na), sum))==0), sum(unlist(lapply(lapply(ifitb3.bs, is.na), sum))==0))/it
)

write.csv(bootconv, file="bootconv.csv")

#====================================================================
# Stocks used in each indicator
#====================================================================

df0 <- melt(saeu[!saeu$projected,], c('FishStock', 'Year'), c('sfFind', 'sfSBL', 'sfCFP', 'sfI7', 'sfI8'))
df0 <- do.call("rbind", lapply(split(df0, df0$FishStock), function(x) subset(x, Year==max(x$Year))))
# fix year for I10 when assessment not from previous year
df1 <- sfI10
df1$Year <- subset(df0, FishStock %in% df1$FishStock & variable=="sfFind")$Year
# merge
df0 <- merge(df0, df1, all=TRUE)
df0 <- rbind(df0, sfI12)
levels(df0$variable) <- c('above/below Fmsy', 'in/out SBL', 'in/out CFP', 'F/Fmsy trends', 'Biomass trends', 'Decadal recruitment trends', "Biomass data category 3 trends")
stkPerIndicator <- dcast(df0, FishStock+Year~variable, value.var='value')

# NOTE: this file must be fixed "by hand" to remove duplications 
# created for the cat 1 stocks which were projected 
# (no time to right code now ...)
write.csv(stkPerIndicator, file="stkPerIndicator.csv")

#====================================================================
# Coverage
#====================================================================
 
# All stocks of relevance
stocks <- subset(saeu, Year==fnlYear)$FishStock
# All stocks with B indicator
bind_stocks <- subset(saeu, Year==fnlYear & !is.na(indB))$FishStock 
# All stocks with F indicator - Same as stocks
find_stocks <- subset(saeu, Year==fnlYear & !is.na(indF))$FishStock 
# All stocks with Bpa indicator
bpaind_stocks <- subset(saeu, Year==fnlYear & !is.na(indBpa))$FishStock 
# All stocks with Fpa indicator - Same as stocks
fpaind_stocks <- subset(saeu, Year==fnlYear & !is.na(indFpa))$FishStock 

# Current list
all_stocks <- unique(isa12sf$FishStock)
# ignore NA
all_stocks <- all_stocks[!is.na(all_stocks)]

# Which stocks to drop from all stocks
drop_stock <- all_stocks[!(all_stocks %in% stocks)]

# Which stocks to drop as no f indicator
drop_stock_f <- all_stocks[!(all_stocks %in% find_stocks)]

# Which stocks to drop as no b indicator
drop_stock_b <- all_stocks[!(all_stocks %in% bind_stocks)]
    
# Which stocks to drop as no fpa indicator
drop_stock_fpa <- all_stocks[!(all_stocks %in% fpaind_stocks)]

# Which stocks to drop as no bpa indicator
drop_stock_bpa <- all_stocks[!(all_stocks %in% bpaind_stocks)]
    
# Set dropped stocks to NA in FishStock column
isa12sf$FindFishStock <- isa12sf$FishStock
isa12sf[isa12sf$FindFishStock %in% drop_stock_f,"FindFishStock"] <- as.character(NA)
isa12sf$BindFishStock <- isa12sf$FishStock
isa12sf[isa12sf$BindFishStock %in% drop_stock_b ,"BindFishStock"] <- as.character(NA)
isa12sf$FpaindFishStock <- isa12sf$FishStock
isa12sf[isa12sf$FpaindFishStock %in% drop_stock_fpa,"FpaindFishStock"] <- as.character(NA)
isa12sf$BpaindFishStock <- isa12sf$FishStock
isa12sf[isa12sf$BpaindFishStock %in% drop_stock_bpa,"BpaindFishStock"] <- as.character(NA)

# Proportion of TACs that have at least one rectangle assessed by FindFishStock and BindFishStock
outf <- aggregate(isa12sf$FindFishStock, by=list(isa12sf$TAC_id), function(x) {
          no_rect_ass_find <- sum(!is.na(x))
          assessed_find <- no_rect_ass_find > 1
          return(assessed_find)
})

outb <- aggregate(isa12sf$BindFishStock, by=list(isa12sf$TAC_id), function(x) {
          no_rect_ass_bind <- sum(!is.na(x))
          assessed_bind <- no_rect_ass_bind > 1
          return(assessed_bind)
})

outfpa <- aggregate(isa12sf$FpaindFishStock, by=list(isa12sf$TAC_id), function(x) {
          no_rect_ass_find <- sum(!is.na(x))
          assessed_find <- no_rect_ass_find > 1
          return(assessed_find)
})

outbpa <- aggregate(isa12sf$BpaindFishStock, by=list(isa12sf$TAC_id), function(x) {
          no_rect_ass_bind <- sum(!is.na(x))
          assessed_bind <- no_rect_ass_bind > 1
          return(assessed_bind)
})

coverage <- data.frame(
	No_stocks = c(length(find_stocks), length(bind_stocks), length(fpaind_stocks), length(bpaind_stocks)),
	No_TACs = length(unique(isa12sf$TAC_id)),
	No_TACs_assessed = c(sum(outf$x), sum(outb$x), sum(outfpa$x), sum(outbpa$x)),
	Frac_TACs_assessed = c(mean(outf$x),mean(outb$x), mean(outfpa$x), mean(outbpa$x))
)
rownames(coverage) <- c("F_indicator", "B_indicator", "Fpa_indicator", "Bpa_indicator")

write.csv(coverage, "coverage.csv")

# number of stocks for which MSYBtrigger==Bpa
#df0 <- transform(saeu, bb=Bpa/MSYBtrigger==1)
#length(unique(subset(df0, bb==TRUE)$FishStock))

#====================================================================
# Exporting and saving
#====================================================================

write.csv(saeu, file="saeu.csv")
save.image("RData_nea")

